from vcCommand import *
from vcHelpers.Application import *
import os, vcVector, vcMatrix
from datetime import datetime
import subprocess, sys
from spatialmedia import metadata_utils

app = getApplication()
sim = getSimulation()

def console(contents):
  print contents
  
def OnOpenFolder(arg):
  os.startfile(rootFolder)       
  
def resetViews():
  for viewNameMatrix in viewNameMatrixs:
    view = app.findView(viewNameMatrix)
    view.CameraMatrix = viewNameMatrixs[viewNameMatrix]
    view.CameraFovy = 35.0      

def modifyViews():
  for view in app.Views:  
    if view.Name == "CurrentView":
      continue
    cameraMatrix = view.CameraMatrix
    cameraMatrix.invert()
    cameraMatrix.setWPR(LFRDBU[repeatTime][1], LFRDBU[repeatTime][2], LFRDBU[repeatTime][3])
    cameraMatrix.invert()
    view.CameraMatrix = cameraMatrix
    view.CameraFovy = 90.0

def updateCurrentView():
  view = app.findView("CurrentView")
  if view:
    app.deleteView("CurrentView")
  app.createView("CurrentView")

def deleteTempFiles():
  tempFilenames = ["Left", "Front", "Right", "Down", "Back", "Up", "Stack", "EAC"]
  for fileName in os.listdir(rootFolder):
    for tempFilename in tempFilenames:
      if tempFilename in fileName:
        os.remove("%s\\%s" % (rootFolder, fileName))
        break
  
def recording():
  global recordingPaths
  app.ProgressStatus = "Recording - " + LFRDBU[repeatTime][0]
  app.ProgressValue += 0.1 
  app.Recorder = VC_RECORDER_VIDEO  
  app.RecordingShow = False
  app.RecordScreen = False
  app.RecordingWidth = size.Value
  app.RecordingHeight = size.Value
  app.RecordingFrameRate = 30  
  app.RecordingFileName = "%s\\%s" % (rootFolder, LFRDBU[repeatTime][0])
  recordingPaths.append(app.RecordingFileName)
  app.Recording = True
  
def WhenStartStop(simulation, isStart): 
  global repeatTime
  
  if isStart == False:
    # Sim time to start time, start recording
    if sim.SimTime == startTime.Value:
      sim.SimSpeed = simSpeed.Value # VC 4.7 Bug report 25026
      app.startSimulation(endTime.Value)
      recording()
    # Sim time to end time, stop recording
    elif sim.SimTime == endTime.Value:
      app.resetSimulation()
      app.Recording = False      
      repeatTime += 1 
      
      # Continue to next direction
      if repeatTime < 6:  
        modifyViews()
        sim.restoreInitialState()                 
        sim.SimSpeed = 1000.0 # Before recording, run simulation as much as possible
        app.startSimulation(startTime.Value)  # Run to start time without recording
      # 6 directions finished
      else:
        sim.OnStartStop = None          
        resetViews()
        app.useView("CurrentView")
        app.deleteView("CurrentView")     
        
        app.ProgressStatus = "Stacking, EACing and injecting ..."  
        app.ProgressValue = 0.9 
        
        # Dunno how to assign video extension in Python API yet
        extension = ".avi"
        if os.path.exists(recordingPaths[0] + ".mp4"):
          extension = ".mp4"
        elif os.path.exists(recordingPaths[0] + ".mov"):
          extension = ".mov"
        
        # Stack 6 videos
        stackFilePath = "%s\\Stack.mp4" % (rootFolder)
        stackCommand = "\"%s\" -loglevel quiet -stats -y -i \"%s%s\" -i \"%s%s\" -i \"%s%s\" -i \"%s%s\" -i \"%s%s\" -i \"%s%s\" -filter_complex \"[0:v][1:v][2:v]hstack=inputs=3[top];[3:v][4:v][5:v]hstack=inputs=3[bottom];[top][bottom]vstack=inputs=2[v]\" -map [v] \"%s\"" % (ffmpegPath, recordingPaths[0], extension, recordingPaths[1], extension, recordingPaths[2], extension, recordingPaths[3], extension, recordingPaths[4], extension, recordingPaths[5], extension, stackFilePath)                  
        subprocess.call(stackCommand)
        
        # EAC
        eacFilePath = "%s\\EAC.mp4" % (rootFolder)
        eacCommand = "\"%s\" -loglevel quiet -stats -y -i \"%s\" -vf \"v360=c3x2:e:cubic:in_forder='lfrdbu':in_frot='000000'\" \"%s\"" % (ffmpegPath, stackFilePath, eacFilePath)
        subprocess.call(eacCommand)
        
        # Inject
        currentTime = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        finalFilePath = "%s\\%s.mp4" % (rootFolder, currentTime)
        metadata = metadata_utils.Metadata()
        metadata.video = metadata_utils.generate_spherical_xml()
        metadata_utils.inject_metadata(eacFilePath, finalFilePath, metadata, console)            
        
        sim.SimSpeed = 1.0 # Set sim speed to 1.0
        app.ProgressStatus = "Done"  
        app.ProgressValue = 1.0
        
        deleteTempFiles()
        os.startfile(finalFilePath)
        
def OnCreate(arg):  
  global repeatTime, viewNameMatrixs, recordingPaths
  
  app.resetSimulation()
  
  # Check external ffmpeg file, internal ffmpeg file should be alwasy there
  if not os.path.exists(ffmpegPath):
    commandFolder = getCommandPath().split("\\")[:-1]     
    commandFolder = "\\".join(commandFolder)
    app.messageBox("Please go to\n\nhttps://www.gyan.dev/ffmpeg/builds/#release-builds to download latest ffmpeg-release-essentials.zip\n\nUnzip \\bin\\ffmpeg.exe to %s" % commandFolder[8:], "", 2, 1)    
    return  
  
  # Check if Camera Aniator has any statement
  viewAnimation = app.findLayoutItem("ViewAnimation") 
  viewAnimationRoutine = viewAnimation.Program.MainRoutine
  if len(viewAnimationRoutine.Statements) == 0:       
    app.messageBox("Please add at least one statement in Camera Animator", "", 2, 1)    
    return
    
  app.ProgressStatus = "Save current views ..."  
  app.ProgressValue = 0.1
  viewNameMatrixs = {}
  for view in app.Views:        
    viewNameMatrixs[view.Name] = view.CameraMatrix   
  updateCurrentView()
  
  app.ProgressStatus = "Initializing ..."  
  app.ProgressValue = 0.2
  recordingPaths = []
  repeatTime = 0  
  modifyViews() 
  
  sim.OnStartStop = WhenStartStop
  sim.SimSpeed = 1000.0 # Before recording, run simulation as much as possible
  sim.setInitialState()    
  app.startSimulation(startTime.Value) # Run to start time without recording

def Onffmpeg(arg):
  global ffmpegPath
  if ffmpeg.Value == "Internal":
    ffmpegPath = sys.executable.split("\\")[:-1]  
    ffmpegPath.append("ffmpeg")
    ffmpegPath.append("ffmpeg.exe")
    ffmpegPath = "\\".join(ffmpegPath)
    print "Due to ffmpeg version, internal ffmpeg may not be able to generate the correct video, has submitted a request to VC for upgrading the ffmpeg version, thank you, support and development team"
  elif ffmpeg.Value == "External":
    ffmpegPath = getCommandPath().split("\\")[:-1]  
    ffmpegPath.append("ffmpeg.exe")
    ffmpegPath = "\\".join(ffmpegPath)
    ffmpegPath = ffmpegPath[8:]
    
    if not os.path.exists(ffmpegPath):
      commandFolder = getCommandPath().split("\\")[:-1]     
      commandFolder = "\\".join(commandFolder)
      app.messageBox("Please go to\n\nhttps://www.gyan.dev/ffmpeg/builds/#release-builds to download latest ffmpeg-release-essentials.zip\n\nUnzip \\bin\\ffmpeg.exe to %s" % commandFolder[8:], "", 2, 1)    
  
  print "------------ ffmpeg path: " + ffmpegPath
  
def OnStart():
  global diag, ffmpeg, size, rootFolder, simSpeed, startTime, endTime, LFRDBU
  LFRDBU = []
  LFRDBU.append(["Left", 90.0, 0.0, 90.0])
  LFRDBU.append(["Front", 90.0, 0.0, 0.0])
  LFRDBU.append(["Right", 90.0, 0.0, -90.0])
  LFRDBU.append(["Down", 0.0, 0.0, 0.0])
  LFRDBU.append(["Back", 90.0, 0.0, 180.0])
  LFRDBU.append(["Up", 180.0, 0.0, 0.0])
  
  # Check root folder
  rootFolder = getCommandPath().split("\\")[:-1]    
  rootFolder.append("Videos")
  rootFolder = "\\".join(rootFolder)
  rootFolder = rootFolder[8:]
  if not os.path.exists(rootFolder):
    os.makedirs(rootFolder)
  
  # Diag
  diag = getDialog(Caption = "YouTube360")
  ffmpeg = diag.addProperty("ffmpeg", VC_STRING, Onffmpeg, VC_PROPERTY_STEP)
  ffmpeg.StepValues = ["Internal", "External"]
  ffmpeg.Value = "External"
  size = diag.addProperty("Size", VC_INTEGER, None)
  size.Value = 2000  
  simSpeed = diag.addProperty("SimSpeed", VC_REAL, None) # VC 4.7 Bug report 25026
  simSpeed.Value = 5.0 # VC 4.7 Bug report 25026
  startTime = diag.addProperty("StartTime (s)", VC_REAL, None)
  startTime.Value = 5.0
  endTime = diag.addProperty("EndTime (s)", VC_REAL, None)
  endTime.Value = 10.0
  create = diag.addProperty("Create", VC_BUTTON, OnCreate)
  openFolder = diag.addProperty("Folder", VC_BUTTON, OnOpenFolder)
  
  diag.show()